<?php

//  Copyright (C) 2002-2006 Balint Kis (balint-i-s.net)
//  Copyright (C) 2005-2006 Solomon Peachy (pizza.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "exifer/exif.php";

function validate_exif_string($exif_string) {
  $string_length = strlen($exif_string);
  $newstring = "";
  $j = 0;

  /* Remove non displayable ASCII characters */
  for ($i=0; $i < $string_length; $i++) {
    $ascii_num = ord($exif_string[$i]);

    if ($ascii_num == 0x27)
      continue;

    if ((0xff > $ascii_num) && ($ascii_num > 0x1F)) {
      $newstring = $newstring . $exif_string[$i];
      $j++;
    }
  }

  return pg_escape_string($newstring);
}


function replace_resolution_unit($string, $resolution_unit) {
  $trans = array("ResolutionUnit" => $resolution_unit);
  return strtr($string, $trans);
}

function convert_exif2sql_timestamp($timestamp) {
  if (!$timestamp)
    return "";

  $year   = substr($timestamp,  0, 4);
  $month  = substr($timestamp,  5, 2);
  $day    = substr($timestamp,  8, 2);
  $hour   = substr($timestamp, 11, 2);
  $minute = substr($timestamp, 14, 2);
  $second = substr($timestamp, 17, 2);

  if ($year == 0 || $month == 0 || $day == 0 || $hour == "" || $minute == "" || $second == "") {
    $today = getdate();
    return sprintf("%04d-%02d-%02d %02d:%02d:%02d", $today['year'], $today['mon'], $today['mday'], $today['hours'], $today['minutes'], $today['seconds']);
  }

  $sql_timestamp = $year.'-'.$month.'-'.$day.' '.$hour.':'.$minute.':'.$second;

  return $sql_timestamp;
}

/* Convert EXIF ratios (eg 60/10, 640/32) into a real number */
function convert_exif_ratio($ratio) {
  $pos = strpos($ratio, "/");
  if ($pos != false) {
    $value = substr($ratio, 0, $pos);
    $value /= substr($ratio, $pos + 1);
    $ratio = $value;
  }
  return $ratio;
}

function convert_exif_aperture($database, $aperture) {
  $aperture_id = "null";
  $minimum_distance = 10000;

  $all_apertures = pg_query($database, "select identifier, value from aperture");
  for ($item = 0; $item < pg_num_rows($all_apertures); $item++) {
    $current = pg_fetch_row($all_apertures, $item);
    $distance = abs($current[1] - $aperture);
    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $aperture_id = $current[0];
    }
    if ($minimum_distance == 0)
      break;
  }
  return $aperture_id;
}

function convert_exif_shutter($database, $shutter) {
  $pos = strpos($shutter, "/");
  if ($pos != false) {
    $effective_shutter = substr($shutter, 0, $pos);
    $effective_shutter /= substr($shutter, $pos+1);
  } else {
    $effective_shutter = $shutter;
  }

  $shutter_id = "null";
  $minimum_distance = 10000;

  $all_speeds = pg_query($database, "select identifier, value from shutter");
  for ($item = 0; $item < pg_num_rows($all_speeds); $item++) {
    $current = pg_fetch_row($all_speeds, $item);

    $pos = strpos($current[1], "/");
    if ($pos === false) {
      $distance = abs($current[1] - $effective_shutter);
    }
    else {
      $effective_tmp_shutter = substr($current[1], 0, $pos);
      $effective_tmp_shutter /= substr($current[1], $pos + 1);
      $distance = abs($effective_shutter - $effective_tmp_shutter);
    }

    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $shutter_id = $current[0];
    }

    if ($minimum_distance == 0)
      break;
  }
  return $shutter_id;
}

function convert_exif_ev_comp($database, $ev_comp) {
  $ev_comp_id = "null";
  $minimum_distance = 10000;

  $all_ev_comps = pg_query($database, "select identifier, value from ev_comp");
  for ($item = 0; $item < pg_num_rows($all_ev_comps); $item++) {
    $current = pg_fetch_row($all_ev_comps, $item);
    $distance = abs($current[1] - $ev_comp);
    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $ev_comp_id = $current[0];
    }
    if ($minimum_distance == 0)
      break;
  }

  return $ev_comp_id;
}

function convert_exif_focal_length($database, $focal_length) {
  $focal_length_id = "null";
  $minimum_distance = 10000;

  $all_focal_lengths = pg_query($database, "select identifier, value from focal_length");
  for ($item = 0; $item < pg_num_rows($all_focal_lengths); $item++) {
    $current = pg_fetch_row($all_focal_lengths, $item);
    $distance = abs($current[1] - $focal_length);
    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $focal_length_id = $current[0];
    }
    if ($minimum_distance == 0)
      break;
  }
  return $focal_length_id;
}

function convert_exif_iso_film($database, $user_id, $iso) {
  $all_digital_films = pg_query($database, "
     select film.identifier, film_iso.value
       from film, film_type, film_format, film_iso
      where film.users = $user_id
        and film.type = film_type.identifier
        and film_type.format = film_format.identifier
        and film_format.value = 'Digital'
        and film_type.iso = film_iso.identifier");

  /* unlike elsewhere w look for exact matches */
  for ($item = 0; $item < pg_num_rows($all_digital_films); $item++) {
    $current = pg_fetch_row($all_digital_films, $item);
    if ($current[1] == $iso)
      return $current[0];
  }

  return "null";
}

function convert_exif_camera($database, $user_id, $camera_make, $camera_model) {
  $camera = pg_query($database, "
     select camera.identifier
       from camera, camera_type, manufacturer
      where '$camera_make' ~*  manufacturer.name
        and '$camera_model' ~*  camera_type.model
        and camera_type.identifier = camera.type
        and camera_type.manufacturer = manufacturer.identifier
        and camera.users = $user_id");

  if (!$camera)
    return "null";

  if (pg_num_rows($camera) > 0) {
    $this_camera = pg_fetch_row($camera, 0);
    return $this_camera[0];
  }

  return "null";
}

function convert_exif_camera_profile($database, $user_id, $camera_make, $camera_model) {
  $camera = pg_query($database, "
     select camera_type.raw_icc_profile
       from camera, camera_type, manufacturer
      where '$camera_make' ~*  manufacturer.name
        and '$camera_model' ~*  camera_type.model
        and camera_type.identifier = camera.type
        and camera_type.manufacturer = manufacturer.identifier
        and camera.users = $user_id");

  if (!$camera)
    return "null";

  if (pg_num_rows($camera) > 0) {
    $this_camera = pg_fetch_row($camera, 0);
    return $this_camera[0];
  }

  return "null";
}

function exif_string_helper(&$image_data, $exif_data, $image_key, $exif_key)
{
  $tmp = pg_escape_string($exif_data[$exif_key]);
  if (strlen($tmp)) {
    $image_data[$image_key] = $tmp;
  } 
}

function photo_parse_exif($database, $index, $user_id, $image_data) {
  global $sys_exiftool;

  /* Try to use exiftool first */
  if (is_executable($sys_exiftool)) {
    $handle = popen("$sys_exiftool " . $image_data['file'][$index]['name_tmp'], "r");
    if ($handle) {
      $image_data["exif"] = "";
      while (!feof($handle)) {
	$line = fgets($handle);
	$parts = explode(":", $line, 2);
	$parts[0] = trim($parts[0]);
	$parts[1] = trim($parts[1]);
	if ($parts[0] == "File Name") continue;
	if ($parts[0] == "Directory") continue;
	if (substr($parts[1], 2, 6) == "Binary") continue;
	//	print "<li>key: $parts[0] value: $parts[1]";
	//	$image_data["exif"] = $image_data["exif"] . "$line";
	$image_data["exif"] = $image_data["exif"] . "<tr><td>$parts[0]</td><td>$parts[1]</td></tr>";
	$exif_data[$parts[0]] = $parts[1];
      }
      pclose($handle);
      $image_data["exif"] = pg_escape_string($image_data["exif"]);
      $image_data["exif"] = "";  // XXX don't store exif data.

      /* Now we parse the data */
      exif_string_helper($image_data, $exif_data, "author", "Artist");
      exif_string_helper($image_data, $exif_data, "copyright", "Copyright");
      if (!$image_data['IgnoreExifUserComment'])
	exif_string_helper($image_data, $exif_data, "title", "User Comment");

      $image_data["date_of_exposure"] = convert_exif2sql_timestamp($exif_data["Date/Time Original"]);
      if ($image_data["date_of_exposure"] == "")
	$image_data["date_of_exposure"] = convert_exif2sql_timestamp($exif_data["Create Date"]);

      $image_data["camera"] = convert_exif_camera($database, $user_id, validate_exif_string($exif_data["Make"]), validate_exif_string($exif_data["Camera Model Name"]));
      $image_data["camera_input_profile"] = convert_exif_camera_profile($database, $user_id, validate_exif_string($exif_data["Make"]), validate_exif_string($exif_data["Camera Model Name"]));
      
      $image_data["film"] = convert_exif_iso_film($database, $user_id, $exif_data["ISO"]);

      $image_data["flash_comp"] = convert_exif_ev_comp($database, $exif_data["Flash Exposure Compensation"]);

      $tmp = $exif_data["Focal Length"];
      $tmp = substr($tmp, 0, strpos($tmp, 'mm'));

      if ($exif_data["Scale Factor To 35mm Equivalent"] > 2) {
	if ($exif_data["Focal Length In 35mm Format"]) {
	  $tmp = $exif_data["Focal Length In 35mm Format"];
	  $tmp = substr($tmp, 0, strpos($tmp, 'mm'));
	} else {
	  $tmp = $tmp * $exif_data["Scale Factor To 35mm Equivalent"];
	}
      }

      $image_data["focal_length"] = convert_exif_focal_length($database, $tmp);

      $image_data["exp_comp"] = convert_exif_ev_comp($database, $exif_data["Exposure Compensation"]);
      $image_data["exp_diff"] = convert_exif_ev_comp($database, $exif_data["Exposure Difference"]);
      
      $image_data["aperture"] = convert_exif_aperture($database, $exif_data["F Number"]);
      $image_data["shutter"] = convert_exif_shutter($database, $exif_data["Shutter Speed"]);

      /* Color Spaces! */
      if ($image_data["ICC Profile"]) 
	$image_data["colorspace"] = 99; /* Embedded Profile */
      else if ($image_data["Profile Class"]) 
	$image_data["colorspace"]; /* Embedded Profile */
      else if (substr_compare($image_data['file'][$index]['original_name'], "_", 0, 1) == 0)
	$image_data["colorspace"]; /* Adobe RGB */
      else if ($exif_data["Color Space"]) {
	$image_data["colorspace"] = 1; 	/* Default to sRGB */
	if ($exif_data["Color Space"] == "Adobe RGB")
	  $image_data["colorspace"] = 2; /* Adobe RGB */
	else if ($exif_data["Color Space"] != "sRGB")
	  $image_data["colorspace"] = 1; /* Anything other than 'sRGB' is undefined, but we have to use it anway... */
      }

      /* Orientation -- Map the new to the old */
      switch ($exif_data["Orientation"]) {
      case 'Mirror horizontal':  /* 2 */
	$image_data["orientation"] = "Mirrored";
	break;
      case 'Rotate 180':  /* 3 */
	$image_data["orientation"] = "Upsidedown";
	break;
      case 'Mirror vertical':  /* 4 */
	$image_data["orientation"] = "Upsidedown Mirrored";
	break;
      case 'Mirror horizontal and rotate 270 CW': /* 5 */
	$image_data["orientation"] = "90 deg CW Mirrored";
	break;
      case 'Rotate 90 CW': /* 6 */
	$image_data["orientation"] = "90 deg CW";
	break;
      case 'Mirror horizontal and rotate 90 CW': /* 7 */
	$image_data["orientation"] = "90 deg CCW Mirrored";
	break;
      case 'Rotate 270 CW': /* 8 */
	$image_data["orientation"] = "90 deg CCW";
	break;
      case 'Horizontal (normal)': /* 1 */
      default:
	$image_data["orientation"] = "Normal (0 deg)";
	break;
      }

      // New:  "metering" "program" "flash" and "flash mode" 

      return $image_data;
    } 
  } 

  /* Fall back on internal EXIFER code */

  $exif = @read_exif_data_raw($image_data['file'][$index]['name_tmp'], 0);

  if ($exif === FALSE)
    return $image_data;

  print "<li>Extracting EXIF data ... ";

  /* first go through all text fields and remove undisplayable characters */

  if (strlen(validate_exif_string($exif["IFD0"]["Artist"]))) {
    $image_data["author"] = validate_exif_string($exif["IFD0"]["Artist"]);
  }

  $title = validate_exif_string($exif["SubIFD"]["UserComment"]);
  if ($title == "")
    $title = validate_exif_string($exif["IFD0"]["UserCommentOld"]);
  if ($title == "")
    $title = validate_exif_string($exif["SubIFD"]["UserCommentOld"]);

  if(strlen($title) && !$image_data['IgnoreExifUserComment'])
    $image_data["title"] = $title;


  if (strlen(validate_exif_string($exif["IFD0"]["Copyright"]))) {
    $image_data["copyright"] = validate_exif_string($exif["IFD0"]["Copyright"]);
  }

  $image_data["date_of_exposure"] = "";
    $image_data["date_of_exposure"] = convert_exif2sql_timestamp($exif["SubIFD"]["DateTimeOriginal"]);
  if ($image_data["date_of_exposure"] == "")
    $image_data["date_of_exposure"] = convert_exif2sql_timestamp($exif["SubIFD"]["DateTimedigitized"]);
  if ($image_data["date_of_exposure"] == "")
    $image_data["date_of_exposure"] = convert_exif2sql_timestamp($exif["IFD0"]["DateTime"]);

  $image_data["camera"] = convert_exif_camera($database, $user_id, validate_exif_string($exif["IFD0"]["Make"]), validate_exif_string($exif["IFD0"]["Model"]));
  $image_data["camera_input_profile"] = convert_exif_camera_profile($database, $user_id, validate_exif_string($exif["IFD0"]["Make"]), validate_exif_string($exif["IFD0"]["Model"]));

  $image_data["film"] = convert_exif_iso_film($database, $user_id, $exif["SubIFD"]["ISOSpeedRatings"]);

  if ($image_data["film"] == "null")
    $image_data["film"] = convert_exif_iso_film($database, $user_id, $exif["SubIFD"]["MakerNote"]["Settings 1"]["ISO"]);
  if ($image_data["film"] == "null")
    $image_data["film"] = convert_exif_iso_film($database, $user_id, $exif["SubIFD"]["MakerNote"]["ISOSetting"]);

  //  $image_data["iso_override"] = convert_exif_iso_film($database, $user_id, $exif["SubIFD"]["MakerNote"]["Settings 1"]["ISO"]);
  //  $image_data["film"] = convert_exif_iso_film($database, $user_id, $exif["SubIFD"]["MakerNote"]["ISOSetting"]);

  $flash_compensation = $exif["SubIFD"]["MakerNote"]["FlashExposureComp"];
  if (!$flash_compensation)
    $flash_compensation = $exif["SubIFD"]["MakerNote"]["Settings 4"]["FlashBias"];
  if ($flash_compensation) {
    $flash_compensation = substr($flash_compensation, 0, strpos($flash_compensation, "EV"));
    $image_data["flash_comp"] = convert_exif_ev_comp($database, convert_exif_ratio($flash_compensation));
  }

  $fov_mult = "";

  if (eregi("2.2",$exif["SubIFD"]["ExifVersion"]) || $exif["SubIFD"]["FocalLength_35mm"]) {
    $image_data["focal_length"] = convert_exif_focal_length($database, convert_exif_ratio($exif["SubIFD"]["FocalLength_35mm"]));
  } else {
    if (convert_exif_ratio($exif["SubIFD"]["FocalPlaneXResolution"])) {
      $fov_mult = $exif["IFD1"]["ImageWidth"] /
        convert_exif_ratio($exif["SubIFD"]["FocalPlaneXResolution"]);
      if ($exif["SubIFD"]["FocalPlaneResolutionUnit"] == 2) {
        $fov_mult *= 25; // inches
      } else {
        $fov_mult *= 10; // cm
      }
    }

    if ($exif["IFD0"]["Model"] = "E775")
      $fov_mult = 6;

    if (!$fov_mult)
      $fov_mult = 35;

    $fov_mult = 35 / $fov_mult;
    $image_data["focal_length"] = convert_exif_focal_length($database, $exif["SubIFD"]["FocalLength"] * $fov_mult);
  }

  $image_data["aperture"] = convert_exif_aperture($database, substr($exif["SubIFD"]["FNumber"], 2));
  if ($image_data["aperture"] == "null")
    $image_data["aperture"] = convert_exif_aperture($database, substr($exif["SubIFD"]["ApertureValue"], 2));

  $image_data["shutter"] = convert_exif_shutter($database, convert_exif_ratio($exif["SubIFD"]["ExposureTime"]));
  if ($image_data["shutter"] == "null")
    $image_data["shutter"] = convert_exif_shutter($database, convert_exif_ratio($exif["SubIFD"]["ShutterSpeedValue"]));

  $rawdata = $exif["SubIFD"]["ExposureBiasValue"];
  $rawdata = substr($rawdata, 0, strpos($rawdata, "EV"));
  $image_data["exp_comp"] = convert_exif_ev_comp($database, convert_exif_ratio($rawdata));

  $rawdata = $exif["SubIFD"]["MakerNote"]["ExposureDifference"];
  $rawdata = substr($rawdata, 0, strpos($rawdata, "EV"));
  $image_data["exp_diff"] = convert_exif_ev_comp($database, convert_exif_ratio($rawdata));

  $image_data["orientation"] = $exif["IFD0"]["Orientation"];
  if ($image_data["orientation"] == "")
    $image_data["orientation"] = $exif["IFD1"]["Orientation"];

  $image_data["exif"] = "";
  if (($exif_string = validate_exif_string($exif["GPS"]["GeodeticSurveyData"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Geodetic Survey Data</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["GPS"]["Latitude Reference"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Latitude Reference</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["GPS"]["Latitude"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Latitude</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["GPS"]["Longitude Reference"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Longitude Reference</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["GPS"]["Longitude"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Longitude</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["GPS"]["NumTags"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Number of tags</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["GPS"]["Time"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Time</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["GPS"]["Version"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Version</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["ACDComment"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>ACD Comment</td><td>".$exif_string."</td></tr>";


  if (($exif_string = validate_exif_string($exif["IFD0"]["HostComputer"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Host Computer</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["ImageDescription"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Image Description</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["ImageType"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Image Type</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["PhotometricInterpret"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Photometric Interpretation</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["PhotoshopSettings"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Photoshop Settings</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["PlanarConfig"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Planar Configuration</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["PrimaryChromaticities"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Primary Chromaticities</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["ReferenceBlackWhite"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Reference Black and White</td><td>".$exif_string."</td></tr>";

  if (($exif_resolution_unit = validate_exif_string($exif["IFD0"]["ResolutionUnit"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Resolution Unit</td><td>".$exif_resolution_unit."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["SamplePerPixel"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Sample Per Pixel</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["Software"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Software</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["WhitePoint"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>White Point</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["YCbCrCoefficients"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>YCbCr Coefficients</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["YCbCrPositioning"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>YCbCr Positioning</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD0"]["xResolution"]))) {
    $exif_string = replace_resolution_unit($exif_string, $exif_resolution_unit);
    $image_data["exif"] = $image_data["exif"] . "<tr><td>X Resolution</td><td>".$exif_string."</td></tr>";
  }

  if (($exif_string = validate_exif_string($exif["IFD0"]["yResolution"]))) {
    $exif_string = replace_resolution_unit($exif_string, $exif_resolution_unit);
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Y Resolution</td><td>".$exif_string."</td></tr>";
  }

  if (($exif_string = validate_exif_string($exif["IFD1"]["BitsPerSample"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Bits Per Sample</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["Compression"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Compression</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["ImageLength"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Image Length</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["ImageWidth"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Image Width</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["JpegIFByteCount"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>JPEG IF Byte Count</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["JpegIFOffset"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>JPEG IF Offset</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["PhotometricInterpretation"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Photometric Interpretation</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["PlanarConfiguration"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Planar Configuration</td><td>".$exif_string."</td></tr>";

  if (($exif_resolution_unit = validate_exif_string($exif["IFD1"]["ResolutionUnit"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Resolution Unit</td><td>".$exif_resolution_unit."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["RowsPerStrip"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Rows Per Strip</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["SamplesPerPixel"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Sample Per Pixel</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["StripByteCounts"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Strip Byte Counts</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["StripOffsets"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Strip Offset</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["ThumbnailDescription"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Thumbnail Description</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["ThumbnailMake"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Thumbnail Make</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["ThumbnailModel"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Thumbnail Model</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["ThumbnailOrientation"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Thumbnail Orientation</td><td>".$exif_string."</td></tr>";

  if (($exif_resolution_unit = validate_exif_string($exif["IFD1"]["ThumbnailResolutionUnit"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Thumbnail ResolutionUnit</td><td>".$exif_resolution_unit."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["ThumbnailXResolution"]))) {
    $exif_string = replace_resolution_unit($exif_string, $exif_resolution_unit);
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Thumbnail X Resolution</td><td>".$exif_string."</td></tr>";
  }

  if (($exif_string = validate_exif_string($exif["IFD1"]["ThumbnailYResolution"]))) {
    $exif_string = replace_resolution_unit($exif_string, $exif_resolution_unit);
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Thumbnail Y Resolution</td><td>".$exif_string."</td></tr>";
  }

  if (($exif_string = validate_exif_string($exif["IFD1"]["YCbCrSubSampling"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>YCbCr Sub Sampling</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["IFD1"]["xResolution"]))) {
    $exif_string = replace_resolution_unit($exif_string, $exif_resolution_unit);
    $image_data["exif"] = $image_data["exif"] . "<tr><td>X Resolution</td><td>".$exif_string."</td></tr>";
  }

  if (($exif_string = validate_exif_string($exif["IFD1"]["yResolution"]))) {
    $exif_string = replace_resolution_unit($exif_string, $exif_resolution_unit);
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Y Resolution</td><td>".$exif_string."</td></tr>";
  }

  if (($exif_string = validate_exif_string($exif["InteroperabilityIFD"]["InteroperabilityIndex"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Interoperability Index</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["InteroperabilityIFD"]["InteroperabilityVersion"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Interoperability Version</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["InteroperabilityIFD"]["RelatedImageFileFormat"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Related Image File Format</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["InteroperabilityIFD"]["RelatedImageLength"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Related Image Length</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["InteroperabilityIFD"]["RelatedImageWidth"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Related Image Width</td><td>".$exif_string."</td></tr>";


  if (($exif_string = validate_exif_string($exif["SubIFD"]["BrightnessValue"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Brightness Value</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["CFAPattern"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>CFA Pattern</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ColorSpace"]))) {
    $image_data['colorspace'] = 1; // default to sRGB
    
    if ($exif["InterColorProfile"] == 1) {
      $image_data['colorspace'] = 99; // If there's an embedded profile..
    } else if ($exif["ValidICCData"] == 1) {
      $image_data['colorspace'] = 99; // If there's an embedded profile..
    } else if (substr_compare($image_data['file'][$index]['original_name'], "_", 0, 1) == 0) {
      $image_data['colorspace'] = 2; // AdobeRGB
    } else if ($exif_string != "sRGB") { // raw value is 1
      $image_data['colorspace'] = 1;  // anything other than '1' is "undefined"
      // but we have to treat it as sRGB for consistency's sake.
    } 

    $image_data["exif"] = $image_data["exif"] . "<tr><td>Color Space</td><td>".$exif_string."</td></tr>";
  }
  if (($exif_string = validate_exif_string($exif["SubIFD"]["ComponentsConfiguration"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Components Configuration</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["CompressedBitsPerPixel"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Compressed Bits Per Pixel</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["Contrast"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Contrast</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["CustomerRender"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Customer Render</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["DigitalZoomRatio"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Digital Zoom Ratio</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ExifImageHeight"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>EXIF Image Height</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ExifImageWidth"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>EXIF Image Width</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ExifInteroperabilityOffset"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>EXIF Interoperability Offset</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ExifVersion"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>EXIF Version</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ExposureIndex"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Exposure Index</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ExposureMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Exposure Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ExposureProgram"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Exposure Program</td><td>".$exif_string."</td></tr>";


  if (($exif_string = validate_exif_string($exif["SubIFD"]["FileSource"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>File Source</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["Flash"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Flash</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["FlashEnergy"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Flash Energy</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["FlashPixVersion"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Flash Pix Version</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["GainControl"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Gain Control</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["ImageHistory"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Image History</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["KnownMaker"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Known Maker</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["LightSource"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Light Souce</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["CameraID"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Identifier</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["CameraSerialNumber"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Serial Number</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["CustomFunctions"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Custom Functions</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["DigiZoom"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Digital Zoom</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["FirmwareVersion"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Firmware Version</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["ImageNumber"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Image Number</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["ImageType"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Image Type</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["JpegQual"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera JPEG Quality</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Macro"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Macro</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["OwnerName"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Owner Name</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["AFPointSelected"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Auto Focus Point</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["Bytes"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Bytes</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["Contrast"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Contrast</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["DigitalZoom"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Digital Zoom</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["DriveMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Driver Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["EasyShooting"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Easy Shooting</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["ExposureMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Exposure Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["Flash"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Flash</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["FlashActivity"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Flash Activity</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["FlashDetails"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Flash Details</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["FocalUnits"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Focal Units</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["FocusMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Focus Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["FocusType"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Focus Type</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["ImageSize"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Image Size</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["LongFocalLength"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Long Focal Length</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["Macro"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Macro</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["MeteringMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Metering Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["Quality"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Quality</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["Saturation"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Saturation</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["SelfTimer"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Self Timer</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["Sharpness"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Sharpness</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 1"]["ShortFocalLength"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Short Focal Length</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 4"]["AFPointUsed"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Auto Focus Point Used</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 4"]["Bytes"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Bytes</td><td>".$exif_string."</td></tr>";


  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 4"]["SequenceNumber"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Sequence Number</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 4"]["SubjectDistance"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Subject Distance</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["Settings 4"]["WhiteBalance"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera White Balance</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["SoftwareRelease"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Software Release</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MakerNote"]["SpecialMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Camera Special Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MaxApertureValue"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Maximum Aperture Value</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["MeteringMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Metering Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["RelatedSoundFile"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Related Sound File</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["Saturation"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Saturation</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SceneCaptureMode"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Scene Capture Mode</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SceneType"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Scene Type</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SensingMethod"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Sensing Method</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["Sharpness"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Sharpness</td><td>".$exif_string."</td></tr>";


  if (($exif_string = validate_exif_string($exif["SubIFD"]["SpacialFreqResponse"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Spacial Frequency Response</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SpectralSensitivity"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Spectral Sensitivity</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SubjectDistance"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Subject Distance</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SubjectLocation"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Subject Location</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SubsecTimeDigitized"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Subsec Time Digitiezed</td><td>".$exif_string."</td></tr>";

  if (($exif_string = validate_exif_string($exif["SubIFD"]["SubsecTimeOriginal"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>Subsec Time Original</td><td>".$exif_string."</td></tr>";


  if (($exif_string = validate_exif_string($exif["SubIFD"]["WhiteBalance"])))
    $image_data["exif"] = $image_data["exif"] . "<tr><td>White Balance</td><td>".$exif_string."</td></tr>";

  // I don't want any of this crap stored.
  $image_data["exif"] = "";

  print "done</li>";
  return $image_data;
}

?>
